package bg.bas.dcl.entities;

import java.util.ArrayList;
import java.util.HashMap;

public class MultilingualLexicalEntry extends HashMap<String, ArrayList<Lemma>>{
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	// key - language code; value - list of synonyms or variants

	public ArrayList<String> domainCategory = new ArrayList<String>();

	public MultilingualLexicalEntry () {
		
	}
	
	public void setLanguageEntry (String langCode, ArrayList<Lemma> list) {
		put(langCode.toUpperCase(), list);
	}
	
	public void setLanguageEntry (String langCode, Lemma l) {

		ArrayList<Lemma> list = new ArrayList<Lemma>();
		list.add(l);
		put(langCode.toUpperCase(), list);
	}

	public void addToLanguageEntry (String langCode, Lemma l) {

		ArrayList<Lemma> list;
		
		if (containsKey(langCode)) list = get(langCode.toUpperCase());
		else list = new ArrayList<Lemma>();
		
		if (!list.contains(l)) list.add(l);
		put(langCode.toUpperCase(), list);
	}
	
	
	public void addCategory (String cat) {

		if (!cat.equals("") && !domainCategory.contains(cat))
						domainCategory.add(cat);
	}

	
	public boolean isEmpty (String langCode) {

		if (!containsKey(langCode.toUpperCase())) return true;
		
		ArrayList<Lemma> list = get(langCode.toUpperCase());
		
		if (list.size() == 0) return true;

		return false;
	}
	
	public Lemma getMainLemma (String langCode) {

		if (!containsKey(langCode.toUpperCase())) return null;
		
		ArrayList<Lemma> list = get(langCode.toUpperCase());
		
		if (list.size() == 0) return null;

		return list.get(0);
	}

	
	public String toString(String[] languages){
		String result = "";
		
		for (String lang : languages) {
			
			Lemma lemma = getMainLemma(lang.toUpperCase());
			if (lemma == null) result += "";
			else result += lemma.lemma;
			
			result += "\t";
		}
		
		result = result.substring(0,result.length()-1);
		return result;
	}
	
	public void readEntry (String line, String[] languages){
		String[] data = line.split("\t");
		
		if (data.length != languages.length) return;
		
		for (int i = 0; i<data.length; i++) {
			
			Lemma lemma = new Lemma(data[i]);
			setLanguageEntry(languages[i], lemma);
			
		}
		
		
	}

	
	public String toStringWithCategories(String[] languages){
		String result = toString(languages) + "\t";
		
		for (String cat : domainCategory) {
			
			if (!cat.trim().equals("")) result += cat + "; ";
		}
		
		return result;
	}

	
	public boolean mergeWith(MultilingualLexicalEntry mle){
		boolean result = true;

		for (String key : mle.keySet()) {
			
			Lemma lemma = mle.get(key).get(0);
			Lemma thislemma = new Lemma("");
					
			if (containsKey(key) && get(key).size()>0)	thislemma = get(key).get(0);
			
			if (!lemma.lemma.equals(thislemma.lemma)) {
				
				if (!lemma.lemma.equals("") && !thislemma.lemma.equals("")) result = false;//ok
				
			}
			
		}

		// "PROLEXBASE_TYPE"


		if (!result) {
			return result;
		}
	
		for (String key : mle.keySet()) {
			
			Lemma lemma = mle.get(key).get(0);
			ArrayList<Lemma> thislemmalist = new ArrayList<Lemma>();
			Lemma thislemma = new Lemma("");
			
			if (containsKey(key) && get(key).size()>0)	{
				thislemmalist = get(key);

				thislemma = get(key).get(0);
			}
			
			if (thislemma.lemma.equals("") && !lemma.lemma.equals("")) {
				thislemmalist.add(0, lemma);
				put(key,thislemmalist);
				
			}
			
		}

		
		
		return result;
	}

}
