package bg.bas.dcl.monolingual.en;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Scanner;

public class TokenTaggerEn {
	
	final String apostr = "`’'";
	final String dashes = "-";
	final String numbers = "1234567890";
	final String numPunct = ".,/";
	final String UCletters = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
	final String LCletters = "abcdefghijklmnopqrstuvwxyz";
	
	HashMap<String, String> tagset = new HashMap<String, String>();
	
	public TokenTaggerEn(String tokentagsetfile){
		
		initialise(tokentagsetfile);
	}
	
	public void initialise(String file) {
		try {
			readTagsetFromFile(file);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public void readTagsetFromFile (String tagsetFileName) throws IOException {
		  
	    String NL = System.getProperty("line.separator");
	    Scanner scanner = new Scanner(new File(tagsetFileName), "UTF-8");
	    
	    if (scanner.equals(null)) return;
	    
	    System.out.println("Scanner created");
	    
	    try {
	      while (scanner.hasNextLine()){
	        String rec = scanner.nextLine();
	        
	        rec = rec.trim();
	        if (rec.length() < 2) continue;
	        if (rec.charAt(0) == '#') 
	        	if (rec.substring(1).trim().equalsIgnoreCase("end")) break;
	        	else continue;
	        String[] entry = rec.split("->");
	        if (entry.length != 2) continue;
	        
	        tagset.put(entry[0].trim(),entry[1].trim());
	        

	      }
	      	      
	    }
	    finally{
	     scanner.close(); 
	    }
	}
	
	public String getTokenTag (String token) {
		
		token = token.trim();
		
		if (tagset.containsKey(token)) return tagset.get(token);
		
		boolean firstCapital = false;
		boolean allUpperCase = true;
		boolean allLowerCase = true;
		boolean allLatin = true;
		boolean allNumber = true;
		boolean dash = false;
		boolean ap = false;
		int numPU = 0;
		
		for (int i = 0; i < token.length(); i++) {
			
			String letter = String.valueOf(token.charAt(i));
			
			if (apostr.contains(letter)) {
				ap = true;
				continue;
			}
			if (dashes.contains(letter)) {
				dash = true;
				continue;
			}
			if (numPU==0 && numPunct.contains(letter)) {
				numPU ++;
				continue;
			}


			if (UCletters.contains(letter)) {
				if (i==0) {
					firstCapital = true;
					allNumber = false;
					continue;
				}
				
				allLowerCase = false;
				allNumber = false;
				continue;
			}
	
			if (LCletters.contains(letter)) {
				
				allUpperCase = false;
				allNumber = false;
				continue;
			}
			
			if (numbers.contains(letter)) {
				
//				allUpperCase = false;
//				allLowerCase = false;
				continue;
			}

			System.err.println("Uncovered: " + token);

			allLatin = false;
			
		}
		
		if (!allLatin) return "TOK_M";
		if (numPU < 2 && allNumber) return "TOK_NUM";
		if (numPU > 2 || (numPU>0 && !allNumber)) return "TOK_M";
		
		if (allLowerCase && !firstCapital) {
			if (!ap && !dash) return "TOK_LAT_LCA";
			if (ap && dash) return "TOK_LAT_LCDAA";
			if (dash) return "TOK_LAT_LCDA";
			if (ap) return "TOK_LAT_LCAA";
		}

		if (allUpperCase) {
			if (!ap && !dash) return "TOK_LAT_UCA";
			if (ap && dash) return "TOK_LAT_UCDAA";
			if (dash) return "TOK_LAT_UCDA";
			if (ap) return "TOK_LAT_UCAA";
		}

		if (allLowerCase && firstCapital) {
			if (!ap && !dash) return "TOK_LAT_FUCA";
			if (ap && dash) return "TOK_LAT_FUCDAA";
			if (dash) return "TOK_LAT_FUCDA";
			if (ap) return "TOK_LAT_FUCAA";
		}


		if (!allUpperCase && !allLowerCase) {
			if (!ap && !dash) return "TOK_LAT_MCA";
			if (ap && dash) return "TOK_LAT_MCDAA";
			if (dash) return "TOK_LAT_MCDA";
			if (ap) return "TOK_LAT_MCAA";
		}

		System.err.println("Uncovered: " + token);
		return "TOK_M";
		
	}

}
