package bg.bas.dcl.multilingual.ClauseSplitting.rules;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Scanner;

import bg.bas.dcl.entities.Conjunction;
import bg.bas.dcl.entities.Sentence;
import bg.bas.dcl.entities.VerbForm;
import bg.bas.dcl.entities.Word;
import bg.bas.dcl.multilingual.Chunker.VerbFormRecogniser.ClauseVerb;
import bg.bas.dcl.multilingual.ClauseSplitting.ClauseBorder;
import bg.bas.dcl.multilingual.ClauseSplitting.ClauseSplittingComplexObject;
import bg.bas.dcl.multilingual.ClauseSplitting.ClauseStack;

public class StrongPunctuationClauseSplittingRule implements ClauseSplittingRule {
	
	public boolean isEndingRule = false;
	
	public ArrayList<String> strongPunctuation = new ArrayList<String>();
	
	public StrongPunctuationClauseSplittingRule (String punctExceptionsFile) {

		
		readPunctException(punctExceptionsFile);
		
//		strongPunctuation.add(";");
//		strongPunctuation.add(".");
//		strongPunctuation.add("?");
//		strongPunctuation.add("!");
//		strongPunctuation.add(":");
		strongPunctuation.add("—");
		strongPunctuation.add("…");
	}

	HashMap<String,ArrayList<PunctuationException>> exceptions = new
			HashMap<String,ArrayList<PunctuationException>>();
	void readPunctException(String filename){
		
		try{
			Scanner s = new Scanner(new File(filename));
			
			while (s.hasNextLine()){
				
				
				String line = s.nextLine().trim();
				if (line.equals("")) continue;

				
				String[] data = line.split("\t");
				if (data.length<3) continue;
				
				PunctuationException pe = new PunctuationException();
				pe.position = data[1];
				pe.str = data[0];
				
				if (exceptions.containsKey(data[2])) {
					exceptions.get(data[2]).add(pe);
					
					
				} else {
					ArrayList<PunctuationException> ees = new ArrayList<PunctuationException>();
					ees.add(pe);
					exceptions.put(data[2], ees);
				}
			}
			
			s.close();
		} catch(Exception e) {
			e.printStackTrace();
			System.exit(0);
		}
		
	}
	
	public ClauseSplittingComplexObject findClauseBorders(
			Sentence sentence,
			ArrayList<ClauseVerb> verbs, 
			int spos, int epos,
			ClauseStack stack
			){
		
		ClauseSplittingComplexObject obj = new ClauseSplittingComplexObject();
		obj.cls = stack;
		obj.clborders = new ArrayList<ClauseBorder>();
		
		for (int i=spos; i<epos+1; i++) {
			
			Word t = sentence.get(i);
			if (strongPunctuation.contains(t.wordform) 
					&& !isException(t.wordform, sentence, i)
					) {
				
				if (verbIsPresent(sentence, verbs, 
						obj.cls
							.getIndexFromStack(1),i)){
					while(obj.cls.stackSize()>0){
						obj.clborders.add(new ClauseBorder(i, "CC"));
						obj.cls.removeFromStack();
						
					}		
				}
				
				if (i<sentence.size()-1){
					obj.clborders.add(new ClauseBorder(i+1, "SCOC"));
					obj.cls.putInStack("SCOC", i+1);
				}	
			}
			
		}
		return obj;

		
	}
	
	public boolean isException(String token, Sentence sentence, int tokenpos){
			
//		System.out.println(token + "\t"+sentence.get(tokenpos).wordform);
//		System.out.println(tokenpos  + "\t<\t"+sentence.size());
//		System.out.println(sentence.get(tokenpos+1).wordform.charAt(0)  + "\t"+sentence.get(tokenpos+1).pos);
		

		if (strongPunctuation.contains(sentence.get(tokenpos).wordform)){
			
			if (tokenpos<sentence.size()-1 
					&& (Character.isUpperCase(sentence.get(tokenpos+1).wordform.charAt(0))
							|| sentence.get(tokenpos+1).pos.equals("M"))
					&& !sentence.get(tokenpos+1).pos.startsWith("NH")){
				
				
				return false;
			}
		}
		
		if (exceptions.containsKey(token)) {
			for (PunctuationException pe : exceptions.get(token)){
				if (pe.position.equals("pre")
						&& tokenpos>0){
					if (!pe.str.startsWith("<") 
							&& sentence.get(tokenpos-1).wordform.equals(pe.str)) {
						return true;
					}
					if (pe.str.equals("<l>") 
							&& Character.isLowerCase(sentence.get(tokenpos-1).wordform
								.charAt(sentence.get(tokenpos-1).wordform.length()-1))){
						return true;
					}
					if (pe.str.equals("<u>") 
							&& Character.isUpperCase(sentence.get(tokenpos-1).wordform
								.charAt(sentence.get(tokenpos-1).wordform.length()-1))){
						return true;
					}
				}

			
				if (pe.position.equals("post")
						&& tokenpos<sentence.size()-1){
					if (!pe.str.startsWith("<") 
							&& sentence.get(tokenpos+1).wordform.equals(pe.str)) return true;
					if (pe.str.equals("<l>") 
							&& Character.isLowerCase(sentence.get(tokenpos+1).wordform
								.charAt(0))){
						return true;
					}
					if (pe.str.equals("<u>") 
							&& Character.isUpperCase(sentence.get(tokenpos+1).wordform
								.charAt(0))){
						return true;
					}
				}
			}
		} else {
			return false;
		}
		
		return false;
	}
	
	public boolean isEnding(){
		return isEndingRule;
	}

	
	class PunctuationException{
		String position = "";
		String str = "";
		
		
	}

	public boolean verbIsPresent(Sentence sent, 
			ArrayList<ClauseVerb> verbs, int begin, int end){
		
		
		
		int vind = -1;
		for (VerbForm vf : verbs) {
			if (begin <= vf.get(0).sentpos && vf.get(vf.size()-1).sentpos <= end
					&& sent.get(vf.get(0).sentpos).clauseID.equals(sent.get(begin).clauseID)) {
		
				System.out.println("VC: begin "+begin + " end "+end);
				System.out.println("VC: "+sent.get(vf.get(0).sentpos).wordform 
						+ " "+sent.get(vf.get(0).sentpos).clauseID);
				System.out.println("VC: "+sent.get(begin).wordform + " "+sent.get(begin).clauseID);
				vind = vf.get(vf.size()-1).sentpos;
				break;
			}
		}
		
		if (vind > -1 && vind < end) return true;
		
		return false;
	}

}
