package bg.bas.dcl.semantic.processing;

import java.util.ArrayList;

import bg.bas.dcl.entities.Sentence;
import bg.bas.dcl.entities.TextPosition;
import bg.bas.dcl.entities.VerbForm;
import bg.bas.dcl.entities.Word;
import bg.bas.dcl.multilingual.Chunker.VerbFormRecogniser.VerbComponent;

public class NounPhraseModel extends ArrayList<NounPhraseComponent>{

	String name = "";
	String modelname = "";
	String modelstring = "";
	
	public boolean finalrule = true;
	
	public String getLongName(){
		String result  = "";
		for (NounPhraseComponent c : this){
			result += c.form+(c.form.equals("")?"":",")+c.pos+":"+c.gram+ " ";
		}
		return result;
	}
	
	public int getLength (){
		int len = 0;
		for (NounPhraseComponent c :this){
			if (!c.optional) len++;
		}
		return len;
	}
	
	public NounPhrase findMatch(Sentence sent, int sentID, int start, 
			ArrayList<NounPhraseModel> models) {
		NounPhrase res = new NounPhrase();

		if (start + getLength() > sent.size()) return null;

//		System.out.println("COMPONENT ANALYSIS "
//				+sent.size() + " " + start +" " +getLength());
	
		int optionalCtr = 0;
		int j = 0;
			
		for (int i = start; i<sent.size(); i++) {

			if (j==this.size()) break;
//			System.out.println("COMPONENT "
	//				+j + " " + i);

			Word t = sent.get(i);
			NounPhraseComponent npc = get(j);
			
//			System.out.println("COMPONENT: "+npc.form+","+npc.pos+npc.gram+"\t"+npc.optional);
			
			
			
			if (!npc.optional) {
				
				if (npc.isPhraseModel) {
					NounPhrase subres = null;
					
					for (NounPhraseModel model : models){
						if (model.name.equalsIgnoreCase(npc.form)) {
							
							subres = model.findMatch(sent, sentID, i, 
									models);
							if (subres!=null) break;
						}
					}			
					
					if (subres==null) return null;
					
					res.addAll(subres);
					
					i = subres.get(subres.size()-1).sentpos;
					j++;
					
				} else {
				
					if (!npc.isMatch(t)) return null;
	
				
	
					res.add(new TextPosition(sentID,i));
					if (npc.isHead) {
						res.headPosition = new TextPosition(sentID,i);
						
					}
					
					optionalCtr = 0;
					j++;
				}
			} else {
			
				if (npc.isPhraseModel) {
					NounPhrase subres = null;
					
					for (NounPhraseModel model : models){
						if (model.name.equalsIgnoreCase(npc.form)) {
							
							subres = model.findMatch(sent, sentID, i, 
									models);
							if (subres!=null) break;
						}
					}			
					
					if (subres==null){
						
						j++;
						i--;

						if (optionalCtr<npc.optionalMin) return null;
						else continue;
						
					}
					
					optionalCtr ++;
					
					if (optionalCtr>npc.optionalMax) return null;
					
					res.addAll(subres);
					
					i = subres.get(subres.size()-1).sentpos;
					
					
				} else {

					if (!npc.isMatch(t)) {
						
						j++;
						i--;
	
						if (optionalCtr<npc.optionalMin) return null;
						else continue;
					}
					
					optionalCtr ++;
						
					if (optionalCtr>npc.optionalMax) return null;
					
					res.add(new TextPosition(sentID,i));
				}
				
			}

						
	
			
		}
		if (res.headPosition.sentpos > -1){
//			System.out.println(res.headPosition.sentpos);
			String pos = sent.get(res.headPosition.sentpos).pos;
			
			String gender = "";
			String number = "";
			
			if (pos.contains("s")) number = "s";
			if (pos.contains("p")) number = "p";
	
			if (pos.contains("m")) gender = "m";
			if (pos.contains("n")) gender = "n";
			if (pos.contains("f")) gender = "f";
	
			res.gram = gender + number;
			
			for (TextPosition p : res) {
				if (!checkAgreement(gender,number,sent.get(p.sentpos), 
						p.sentpos<sent.size()-1?sent.get(p.sentpos+1):null)){
					
					return null;
				}
			}
		}
		
		if (res.size()==0) return null;
			
		return res;
	}

	boolean checkAgreement(String gender, String number, Word t, Word next){
		
		if (true) return true;
		
		String[] gens = {"m","n","f"};
		String[] nums = {"s","p"};
		
		if (!t.pos.startsWith("A") && !t.pos.startsWith("B")) return true;
				
//		System.out.println(gender + " " + number + " " + t.wordform+" " + t.pos);

		if (!number.equals("")){
			for (String n : nums) {
				if (!n.equals(number)
						&& t.pos.contains(n)) return false;
			}

		}
		
		if (!gender.equals("") && number.equals("s")){
			for (String g : gens) {
				if (!g.equals(gender)
						&& t.pos.contains(g)) {
					if (next.pos.startsWith("A")
							&& (t.pos.contains("n") 
									|| t.wordform.endsWith("ски"))) {
						t.pos = "DDD"+t.pos;
						return true;
					}
					return false;
				}
			}

		}

		return true;

	}
}
